/* Copyright (c) 2002, Marek Michalkiewicz
   Copyright (c) 2004,2005,2007 Joerg Wunsch
   Copyright (c) 2007  Florin-Viorel Petrov
   Copyright (c) 2008 Carlos Lamas
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.

   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.

   * Neither the name of the copyright holders nor the names of
     contributors may be used to endorse or promote products derived
     from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. */

#ifndef _UTIL_DELAY_H_
#define _UTIL_DELAY_H_ 1

#include <util/delay_basic.h>

/** \file */
/** \defgroup util_delay <util/delay.h>: Convenience functions for busy-wait delay loops
    \code
    #define F_CPU 1000000UL  // 1 MHz
    //#define F_CPU 14.7456E6
    #include <util/delay.h>
    \endcode

    \note As an alternative method, it is possible to pass the
    F_CPU macro down to the compiler from the Makefile.
    Obviously, in that case, no \c \#define statement should be
    used.

    The definitions in this header file are wrappers around the basic
    busy-wait functions from <util/delay_basic.h>.  They are meant as
    convenience functions where actual time values can be specified
    rather than a number of cycles to wait for.  The idea behind is
    that compile-time constant expressions will be eliminated by
    compiler optimization so floating-point expressions can be used
    to calculate the number of delay cycles needed based on the CPU
    frequency passed by the macro F_CPU.

    The functions available allow the specification of microsecond,
    millisecond, and second delays directly, using the application-
    -supplied macro F_CPU as the CPU clock frequency (in Hertz).

    \note Both the delay time and the frequency (F_CPU) <em>must</em> be
    an integer or floating point expression that is a known constant
    at compile-time or the compiler will emit an error.

    Variables with the qualifier 'const' are not always recognized as
    constant values at compile time; that depends on the optimization
    level. With optimizations disabled they are not valid arguments to
    the macros.

*/


#if defined(__DOXYGEN__)


/**
   \ingroup util_delay

   \def _delay_us(__us)

   Perform a delay of \c __us microseconds.

   The macro F_CPU is supposed to be defined to a
   constant defining the CPU clock frequency (in Hertz).

   The maximal possible delay is 25.7698e9 us / F_CPU in MHz.
 */
#define _delay_us(__us)


/**
   \ingroup util_delay

   \def _delay_ms(__ms)

   Perform a delay of \c __ms milliseconds.

   The macro F_CPU is supposed to be defined to a
   constant defining the CPU clock frequency (in Hertz).

   The maximal possible delay is 25.7698e6 ms / F_CPU in MHz.
 */
#define _delay_ms(__ms)


/**
   \ingroup util_delay

   \def _delay_s(__s)

   Perform a delay of \c __s seconds.

   The macro F_CPU is supposed to be defined to a
   constant defining the CPU clock frequency (in Hertz).

   The maximal possible delay is 25.7698e3 s / F_CPU in MHz.
 */
#define _delay_s(__s)


/**
   \ingroup util_delay

   \def _delay_exact_us(__us)

   Perform a delay of \c __us microseconds. The delay time should be
   accurate to +/- 0.5 CPU cycles.

   The macro F_CPU is supposed to be defined to a
   constant defining the CPU clock frequency (in Hertz).

   The maximal possible delay is 25.7698e9 us / F_CPU in MHz.
 */
#define _delay_exact_us(__us)


/**
   \ingroup util_delay

   \def _delay_exact_ms(__ms)

   Perform a delay of \c __ms milliseconds. The delay time should be
   accurate to +/- 0.5 CPU cycles.

   The macro F_CPU is supposed to be defined to a
   constant defining the CPU clock frequency (in Hertz).

   The maximal possible delay is 25.7698e6 ms / F_CPU in MHz.
 */
#define _delay_exact_ms(__ms)


/**
   \ingroup util_delay

   \def _delay_exact_s(__s)

   Perform a delay of \c __s seconds. The delay time should be
   accurate to +/- 0.5 CPU cycles.

   The macro F_CPU is supposed to be defined to a
   constant defining the CPU clock frequency (in Hertz).

   The maximal possible delay is 25.7698e3 s / F_CPU in MHz.
 */
#define _delay_exact_s(__s)


#else  /* !defined(__DOXYGEN__) */


#ifndef F_CPU
/* prevent compiler error by supplying a default */
# warning "F_CPU not defined for <util/delay.h>"
# define F_CPU 1000000UL
#endif


#define _delay_us(__us)                                           \
do {                                                              \
    if (!__builtin_constant_p(F_CPU)) {                           \
        _DELAY_EMIT_ERROR(__STRINGIFY(F_CPU)                      \
                          " is not a constant for _delay_us");    \
    } else if (!__builtin_constant_p(__us)) {                     \
        _DELAY_EMIT_ERROR(__STRINGIFY(__us)                       \
                          " is not a constant for _delay_us");    \
    } else {                                                      \
        _delay_f_us(F_CPU, __us);                                 \
    }                                                             \
} while (0)


#define _delay_ms(__ms)                                           \
do {                                                              \
    if (!__builtin_constant_p(F_CPU)) {                           \
        _DELAY_EMIT_ERROR(__STRINGIFY(F_CPU)                      \
                          " is not a constant for _delay_ms");    \
    } else if (!__builtin_constant_p(__ms)) {                     \
        _DELAY_EMIT_ERROR(__STRINGIFY(__ms)                       \
                          " is not a constant for _delay_ms");    \
    } else {                                                      \
        _delay_f_ms(F_CPU, __ms);                                 \
    }                                                             \
} while (0)


#define _delay_s(__s)                                            \
do {                                                             \
    if (!__builtin_constant_p(F_CPU)) {                          \
        _DELAY_EMIT_ERROR(__STRINGIFY(F_CPU)                     \
                          " is not a constant for _delay_s");    \
    } else if (!__builtin_constant_p(__s)) {                     \
        _DELAY_EMIT_ERROR(__STRINGIFY(__s)                       \
                          " is not a constant for _delay_s");    \
    } else {                                                     \
        _delay_f_s(F_CPU, __s);                                  \
    }                                                            \
} while (0)


#define _delay_exact_us(__us)                                           \
do {                                                                    \
    if (!__builtin_constant_p(F_CPU)) {                                 \
        _DELAY_EMIT_ERROR(__STRINGIFY(F_CPU)                            \
                          " is not a constant for _delay_exact_us");    \
    } else if (!__builtin_constant_p(__us)) {                           \
        _DELAY_EMIT_ERROR(__STRINGIFY(__us)                             \
                          " is not a constant for _delay_exact_us");    \
    } else {                                                            \
        _delay_exact_f_us(F_CPU, __us);                                 \
    }                                                                   \
} while (0)


#define _delay_exact_ms(__ms)                                           \
do {                                                                    \
    if (!__builtin_constant_p(F_CPU)) {                                 \
        _DELAY_EMIT_ERROR(__STRINGIFY(F_CPU)                            \
                          " is not a constant for _delay_exact_ms");    \
    } else if (!__builtin_constant_p(__ms)) {                           \
        _DELAY_EMIT_ERROR(__STRINGIFY(__ms)                             \
                          " is not a constant for _delay_exact_ms");    \
    } else {                                                            \
        _delay_exact_f_ms(F_CPU, __ms);                                 \
    }                                                                   \
} while (0)


#define _delay_exact_s(__s)                                            \
do {                                                                   \
    if (!__builtin_constant_p(F_CPU)) {                                \
        _DELAY_EMIT_ERROR(__STRINGIFY(F_CPU)                           \
                          " is not a constant for _delay_exact_s");    \
    } else if (!__builtin_constant_p(__s)) {                           \
        _DELAY_EMIT_ERROR(__STRINGIFY(__s)                             \
                          " is not a constant for _delay_exact_s");    \
    } else {                                                           \
        _delay_exact_f_s(F_CPU, __s);                                  \
    }                                                                  \
} while (0)


#endif


#endif /* _UTIL_DELAY_H_ */
